@@ -13,17 +13,15 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 
-from __future__ import print_function
+
 
 import sys
 
 if sys.version_info >= ( 3, ):
     import configparser
-    ConfigParser = configparser.ConfigParser
 else:
     #python 2.x
-    import ConfigParser as configparser
-    ConfigParser = configparser.SafeConfigParser
+    import configparser as configparser
 
 import re
 
@@ -89,6 +87,7 @@ class ConfigLineOrder:
         for line in self._lines:
             if line.type == 'section':
                 yield line.name
+        raise StopIteration()
 
 class Line:
     """Base class for all line objects"""
@@ -131,7 +130,7 @@ class SectionLine(Line):
         self._lines.append(CommentLine(line))
 
     def copy(self):
-        return dict(self.items())
+        return dict(list(self.items()))
 
     def items(self):
         return [ (i.name, i.value) for i in self._lines if i.type == 'option' ]
@@ -161,6 +160,7 @@ class SectionLine(Line):
     def __iter__(self):
         for line in self._lines:
             yield line.name
+        raise StopIteration()
 
 
 class CommentLine(Line):
@@ -190,7 +190,7 @@ class OptionLine(Line):
         self.format(line)
 
     def format(self, line):
-        mo = ConfigParser.OPTCRE.match(line.strip())
+        mo = configparser.ConfigParser.OPTCRE.match(line.strip())
         key, val = mo.group('option', 'value')
         self.frmt = line.replace(key.strip(), '%s', 1)
         pos = val.find(' ;')
@@ -203,7 +203,7 @@ class OptionLine(Line):
         return self.value
 
 
-class OscConfigParser(ConfigParser):
+class OscConfigParser(configparser.SafeConfigParser):
     """
     OscConfigParser() behaves like a normal ConfigParser() object. The
     only differences is that it preserves the order+format of configuration entries
@@ -212,7 +212,7 @@ class OscConfigParser(ConfigParser):
     class.
     """
     def __init__(self, defaults={}):
-        ConfigParser.__init__(self, defaults)
+        configparser.SafeConfigParser.__init__(self, defaults)
         self._sections = ConfigLineOrder()
 
     # XXX: unfortunately we have to override the _read() method from the ConfigParser()
@@ -321,7 +321,7 @@ class OscConfigParser(ConfigParser):
             fp.write(str(self))
             fp.write('\n')
         else:
-            ConfigParser.write(self, fp)
+            configparser.SafeConfigParser.write(self, fp)
 
     def has_option(self, section, option, proper=False, **kwargs):
         """
@@ -330,8 +330,8 @@ class OscConfigParser(ConfigParser):
         this section and not "inherited" from the default.
         """
         if proper:
-            return self.optionxform(option) in self._sections[section].keys()
-        return ConfigParser.has_option(self, section, option, **kwargs)
+            return self.optionxform(option) in list(self._sections[section].keys())
+        return configparser.SafeConfigParser.has_option(self, section, option, **kwargs)
 
     # XXX: simplify!
     def __str__(self):
@@ -357,10 +357,4 @@ class OscConfigParser(ConfigParser):
                 ret.append(str(line))
         return '\n'.join(ret)
 
-    def _validate_value_types(self, section="", option="", value=""):
-        if not isinstance(section, str):
-            raise TypeError("section names must be strings")
-        if not isinstance(option, str):
-            raise TypeError("option keys must be strings")
-
 # vim: sw=4 et
