@@ -3,7 +3,7 @@
 # and distributed under the terms of the GNU General Public Licence,
 # either version 2, or (at your option) any later version.
 
-from __future__ import print_function
+
 
 import sys, os
 
@@ -12,32 +12,73 @@ try:
     from urllib.request import HTTPBasicAuthHandler, HTTPCookieProcessor, HTTPPasswordMgrWithDefaultRealm, HTTPError
 except ImportError:
     #python 2.x
-    from urllib import quote_plus
-    from urllib2 import HTTPBasicAuthHandler, HTTPCookieProcessor, HTTPPasswordMgrWithDefaultRealm, HTTPError
+    from urllib.parse import quote_plus
+    from urllib.request import HTTPBasicAuthHandler, HTTPCookieProcessor, HTTPPasswordMgrWithDefaultRealm
+    from urllib.error import HTTPError
 
-from .core import makeurl, streamfile, dgst
-from .grabber import OscFileGrabber, OscMirrorGroup
+from urlgrabber.grabber import URLGrabber, URLGrabError
+from urlgrabber.mirror import MirrorGroup
+from .core import makeurl, streamfile
 from .util import packagequery, cpio
 from . import conf
 from . import oscerr
 import tempfile
 import re
+try:
+    from .meter import TextMeter
+except:
+    TextMeter = None
+
+
+def join_url(self, base_url, rel_url):
+    """to override _join_url of MirrorGroup, because we want to
+    pass full URLs instead of base URL where relative_url is added later...
+    IOW, we make MirrorGroup ignore relative_url
+    """
+    return base_url
+
+
+class OscFileGrabber(URLGrabber):
+    def __init__(self, progress_obj=None):
+        # we cannot use super because we still have to support
+        # older urlgrabber versions where URLGrabber is an old-style class
+        URLGrabber.__init__(self)
+        self.progress_obj = progress_obj
+
+    def urlgrab(self, url, filename, text=None, **kwargs):
+        if url.startswith('file://'):
+            f = url.replace('file://', '', 1)
+            if os.path.isfile(f):
+                return f
+            else:
+                raise URLGrabError(2, 'Local file \'%s\' does not exist' % f)
+        with file(filename, 'wb') as f:
+            try:
+                for i in streamfile(url, progress_obj=self.progress_obj,
+                                    text=text):
+                    f.write(i)
+            except HTTPError as e:
+                exc = URLGrabError(14, str(e))
+                exc.url = url
+                exc.exception = e
+                exc.code = e.code
+                raise exc
+            except IOError as e:
+                raise URLGrabError(4, str(e))
+        return filename
 
-from osc.util.helper import decode_it
-from .meter import create_text_meter
 
 class Fetcher:
     def __init__(self, cachedir='/tmp', api_host_options={}, urllist=[],
-                 http_debug=False, cookiejar=None, offline=False,
-                 enable_cpio=True, modules=[]):
+            http_debug=False, cookiejar=None, offline=False, enable_cpio=True):
         # set up progress bar callback
-        self.progress_obj = None
-        if sys.stdout.isatty():
-            self.progress_obj = create_text_meter(use_pb_fallback=False)
+        if sys.stdout.isatty() and TextMeter:
+            self.progress_obj = TextMeter(fo=sys.stdout)
+        else:
+            self.progress_obj = None
 
         self.cachedir = cachedir
         self.urllist = urllist
-        self.modules = modules
         self.http_debug = http_debug
         self.offline = offline
         self.cpio = {}
@@ -52,6 +93,14 @@ class Fetcher:
             openers += (HTTPCookieProcessor(cookiejar), )
         self.gr = OscFileGrabber(progress_obj=self.progress_obj)
 
+    def failureReport(self, errobj):
+        """failure output for failovers from urlgrabber"""
+        if errobj.url.startswith('file://'):
+            return {}
+        print('Trying openSUSE Build Service server for %s (%s), not found at %s.'
+              % (self.curpac, self.curpac.project, errobj.url.split('/')[2]))
+        return {}
+
     def __add_cpio(self, pac):
         prpap = '%s/%s/%s/%s' % (pac.project, pac.repository, pac.repoarch, pac.repopackage)
         self.cpio.setdefault(prpap, {})[pac.repofilename] = pac
@@ -61,8 +110,6 @@ class Fetcher:
             return
         query = ['binary=%s' % quote_plus(i) for i in pkgs]
         query.append('view=cpio')
-        for module in self.modules:
-            query.append('module=' + module)
         try:
             url = makeurl(apiurl, ['build', project, repo, arch, package], query=query)
             sys.stdout.write("preparing download ...\r")
@@ -77,21 +124,16 @@ class Fetcher:
                     # getbinarylist instead of the public/... route
                     # (which is routed to getbinaries)
                     # getbinaries does not support kiwi builds
-                    if hdr.filename == b'.errors':
+                    if hdr.filename == '.errors':
                         archive.copyin_file(hdr.filename)
                         raise oscerr.APIError('CPIO archive is incomplete '
                                               '(see .errors file)')
                     if package == '_repository':
-                        n = re.sub(b'\.pkg\.tar\.(zst|.z)$', b'.arch', hdr.filename)
-                        if n.startswith(b'container:'):
-                            n = re.sub(b'\.tar\.(zst|.z)$', b'.tar', hdr.filename)
-                            pac = pkgs[decode_it(n.rsplit(b'.', 1)[0])]
-                            pac.canonname = hdr.filename
-                        else:
-                            pac = pkgs[decode_it(n.rsplit(b'.', 1)[0])]
+                        n = re.sub(r'\.pkg\.tar\..z$', '.arch', hdr.filename)
+                        pac = pkgs[n.rsplit('.', 1)[0]]
                     else:
                         # this is a kiwi product
-                        pac = pkgs[decode_it(hdr.filename)]
+                        pac = pkgs[hdr.filename]
 
                     # Extract a single file from the cpio archive
                     try:
@@ -105,20 +147,20 @@ class Fetcher:
                         if os.path.exists(tmpfile):
                             os.unlink(tmpfile)
 
-                for pac in pkgs.values():
+                for pac in list(pkgs.values()):
                     if not os.path.isfile(pac.fullfilename):
                         raise oscerr.APIError('failed to fetch file \'%s\': '
                                               'missing in CPIO archive' %
                                               pac.repofilename)
-        except HTTPError as e:
-            if e.code != 414:
+        except URLGrabError as e:
+            if e.errno != 14 or e.code != 414:
                 raise
             # query str was too large
             keys = list(pkgs.keys())
             if len(keys) == 1:
                 raise oscerr.APIError('unable to fetch cpio archive: '
                                       'server always returns code 414')
-            n = int(len(pkgs) / 2)
+            n = len(pkgs) / 2
             new_pkgs = dict([(k, pkgs[k]) for k in keys[:n]])
             self.__download_cpio_archive(apiurl, project, repo, arch,
                                          package, **new_pkgs)
@@ -127,7 +169,7 @@ class Fetcher:
                                          package, **new_pkgs)
 
     def __fetch_cpio(self, apiurl):
-        for prpap, pkgs in self.cpio.items():
+        for prpap, pkgs in list(self.cpio.items()):
             project, repo, arch, package = prpap.split('/', 3)
             self.__download_cpio_archive(apiurl, project, repo, arch, package, **pkgs)
 
@@ -135,7 +177,8 @@ class Fetcher:
         # for use by the failure callback
         self.curpac = pac
 
-        mg = OscMirrorGroup(self.gr, pac.urllist)
+        MirrorGroup._join_url = join_url
+        mg = MirrorGroup(self.gr, pac.urllist, failure_callback=(self.failureReport, (), {}))
 
         if self.http_debug:
             print('\nURLs to try for package \'%s\':' % pac, file=sys.stderr)
@@ -145,45 +188,35 @@ class Fetcher:
         try:
             with tempfile.NamedTemporaryFile(prefix='osc_build',
                                              delete=False) as tmpfile:
-                mg_stat = mg.urlgrab(pac.filename, filename=tmpfile.name,
+                mg.urlgrab(pac.filename, filename=tmpfile.name,
                            text='%s(%s) %s' % (prefix, pac.project, pac.filename))
-                if mg_stat:
-                    self.move_package(tmpfile.name, pac.localdir, pac)
-
-            if not mg_stat:
-                if self.enable_cpio:
-                    print('%s/%s: attempting download from api, since not found'
-                          % (pac.project, pac.name))
-                    self.__add_cpio(pac)
-                    return
-                print()
-                print('Error: Failed to retrieve %s from the following locations '
-                      '(in order):' % pac.filename, file=sys.stderr)
-                print('\n'.join(pac.urllist), file=sys.stderr)
-                sys.exit(1)
+                self.move_package(tmpfile.name, pac.localdir, pac)
+        except URLGrabError as e:
+            if self.enable_cpio and e.errno == 256:
+                self.__add_cpio(pac)
+                return
+            print()
+            print('Error:', e.strerror, file=sys.stderr)
+            print('Failed to retrieve %s from the following locations '
+                  '(in order):' % pac.filename, file=sys.stderr)
+            print('\n'.join(pac.urllist), file=sys.stderr)
+            sys.exit(1)
         finally:
             if os.path.exists(tmpfile.name):
                 os.unlink(tmpfile.name)
 
     def move_package(self, tmpfile, destdir, pac_obj=None):
         import shutil
-        canonname = None
-        if pac_obj and pac_obj.name.startswith('container:'):
-            canonname = pac_obj.canonname
-        if canonname is None:
-            pkgq = packagequery.PackageQuery.query(tmpfile, extra_rpmtags=(1044, 1051, 1052))
-            if pkgq:
-                canonname = pkgq.canonname()
-            else:
-                if pac_obj is None:
-                    print('Unsupported file type: ', tmpfile, file=sys.stderr)
-                    sys.exit(1)
-                canonname = pac_obj.binary
-        decoded_canonname = decode_it(canonname)
-        if b'/' in canonname or '/' in decoded_canonname:
-            raise oscerr.OscIOError(None, 'canonname contains a slash')
+        pkgq = packagequery.PackageQuery.query(tmpfile, extra_rpmtags=(1044, 1051, 1052))
+        if pkgq:
+            canonname = pkgq.canonname()
+        else:
+            if pac_obj is None:
+                print('Unsupported file type: ', tmpfile, file=sys.stderr)
+                sys.exit(1)
+            canonname = pac_obj.binary
 
-        fullfilename = os.path.join(destdir, decoded_canonname)
+        fullfilename = os.path.join(destdir, canonname)
         if pac_obj is not None:
             pac_obj.canonname = canonname
             pac_obj.fullfilename = fullfilename
@@ -205,21 +238,11 @@ class Fetcher:
         all = len(buildinfo.deps)
         for i in buildinfo.deps:
             i.makeurls(self.cachedir, self.urllist)
-            # find container extension by looking in the cache
-            if i.name.startswith('container:') and i.fullfilename.endswith('.tar.xz'):
-                for ext in ['.tar.xz', '.tar.gz', '.tar']:
-                    if os.path.exists(i.fullfilename[:-7] + ext):
-                        i.canonname = i.canonname[:-7] + ext
-                        i.makeurls(self.cachedir, self.urllist)
-
             if os.path.exists(i.fullfilename):
                 cached += 1
                 if i.hdrmd5:
                     from .util import packagequery
-                    if i.name.startswith('container:'):
-                        hdrmd5 = dgst(i.fullfilename)
-                    else:
-                        hdrmd5 = packagequery.PackageQuery.queryhdrmd5(i.fullfilename)
+                    hdrmd5 = packagequery.PackageQuery.queryhdrmd5(i.fullfilename)
                     if not hdrmd5 or hdrmd5 != i.hdrmd5:
                         os.unlink(i.fullfilename)
                         cached -= 1
@@ -244,12 +267,12 @@ class Fetcher:
                     continue
                 try:
                     # if there isn't a progress bar, there is no output at all
-                    prefix = ''
                     if not self.progress_obj:
                         print('%d/%d (%s) %s' % (done, needed, i.project, i.filename))
-                    else:
-                        prefix = '[%d/%d] ' % (done, needed)
-                    self.fetch(i, prefix=prefix)
+                    self.fetch(i)
+                    if self.progress_obj:
+                        print("  %d/%d\r" % (done, needed), end=' ')
+                        sys.stdout.flush()
 
                 except KeyboardInterrupt:
                     print('Cancelled by user (ctrl-c)')
@@ -267,15 +290,14 @@ class Fetcher:
             dest += '/_pubkey'
 
             url = makeurl(buildinfo.apiurl, ['source', i, '_pubkey'])
-            try_parent = False
             try:
                 if self.offline and not os.path.exists(dest):
                     # may need to try parent
-                    try_parent = True
+                    raise URLGrabError(2)
                 elif not self.offline:
                     OscFileGrabber().urlgrab(url, dest)
                 # not that many keys usually
-                if i not in buildinfo.prjkeys and not try_parent:
+                if i not in buildinfo.prjkeys:
                     buildinfo.keys.append(dest)
                     buildinfo.prjkeys.append(i)
             except KeyboardInterrupt:
@@ -284,16 +306,14 @@ class Fetcher:
                 if os.path.exists(dest):
                     os.unlink(dest)
                 sys.exit(0)
-            except HTTPError as e:
+            except URLGrabError as e:
                 # Not found is okay, let's go to the next project
-                if e.code != 404:
+                if e.errno == 14 and e.code != 404:
                     print("Invalid answer from server", e, file=sys.stderr)
                     sys.exit(1)
-                try_parent = True
 
-            if try_parent:
                 if self.http_debug:
-                    print("can't fetch key for %s" % (i), file=sys.stderr)
+                    print("can't fetch key for %s: %s" % (i, e.strerror), file=sys.stderr)
                     print("url: %s" % url, file=sys.stderr)
 
                 if os.path.exists(dest):
