@@ -1,14 +1,10 @@
 
-from __future__ import print_function
+
 
 import os
 import re
 import struct
 from . import packagequery
-from osc.util.helper import decode_it
-
-def cmp(a, b):
-    return (a > b) - (a < b)
 
 class RpmError(packagequery.PackageError):
     pass
@@ -62,18 +58,11 @@ class RpmQuery(packagequery.PackageQuery, packagequery.PackageQueryResult):
     GREATER = 1 << 2
     EQUAL = 1 << 3
 
-    SENSE_STRONG = 1 << 27
-
     default_tags = (1000, 1001, 1002, 1003, 1004, 1022, 1005, 1020,
         1047, 1112, 1113, # provides
         1049, 1048, 1050, # requires
         1054, 1053, 1055, # conflicts
-        1090, 1114, 1115, # obsoletes
-        1156, 1158, 1157, # oldsuggests
-        5046, 5047, 5048, # recommends
-        5049, 5051, 5050, # suggests
-        5052, 5053, 5054, # supplements
-        5055, 5056, 5057  # enhances
+        1090, 1114, 1115  # obsoletes
     )
 
     def __init__(self, fh):
@@ -118,7 +107,7 @@ class RpmQuery(packagequery.PackageQuery, packagequery.PackageQueryResult):
         data = self.__file.read(self.LEAD_SIZE)
         leadmgc, = struct.unpack('!I', data[:4])
         if leadmgc != self.LEAD_MAGIC:
-            raise RpmError(self.__path, 'not a rpm (invalid lead magic \'%s\')' % leadmgc)
+            raise RpmError(self.__path, 'invalid lead magic \'%s\'' % leadmgc)
         sigtype, = struct.unpack('!h', data[78:80])
         if sigtype != self.HEADERSIG_TYPE:
             raise RpmError(self.__path, 'invalid header signature \'%s\'' % sigtype)
@@ -167,7 +156,7 @@ class RpmQuery(packagequery.PackageQuery, packagequery.PackageQueryResult):
         else:
             raise RpmHeaderError(self.__path, 'unsupported tag type \'%d\' (tag: \'%s\'' % (entry.type, entry.tag))
 
-    def __reqprov(self, tag, flags, version, strong=None):
+    def __reqprov(self, tag, flags, version):
         pnames = self.header.gettag(tag)
         if not pnames:
             return []
@@ -178,24 +167,16 @@ class RpmQuery(packagequery.PackageQuery, packagequery.PackageQueryResult):
             raise RpmError(self.__path, 'cannot get provides/requires, tags are missing')
         res = []
         for name, flags, ver in zip(pnames, pflags, pvers):
-            if strong is not None:
-                # compat code for the obsolete RPMTAG_OLDSUGGESTSNAME tag
-                # strong == 1 => return only "recommends"
-                # strong == 0 => return only "suggests"
-                if strong == 1:
-                    strong = self.SENSE_STRONG
-                if (flags & self.SENSE_STRONG) != strong:
-                    continue
             # RPMSENSE_SENSEMASK = 15 (see rpmlib.h) but ignore RPMSENSE_SERIAL (= 1 << 0) therefore use 14
             if flags & 14:
-                name += b' '
+                name += ' '
                 if flags & self.GREATER:
-                    name += b'>'
+                    name += '>'
                 elif flags & self.LESS:
-                    name += b'<'
+                    name += '<'
                 if flags & self.EQUAL:
-                    name += b'='
-                name += b' %s' % ver
+                    name += '='
+                name += ' %s' % ver
             res.append(name)
         return res
 
@@ -255,24 +236,6 @@ class RpmQuery(packagequery.PackageQuery, packagequery.PackageQueryResult):
     def obsoletes(self):
         return self.__reqprov(1090, 1114, 1115)
 
-    def recommends(self):
-        recommends = self.__reqprov(5046, 5048, 5047)
-        if not recommends:
-            recommends = self.__reqprov(1156, 1158, 1157, 1)
-        return recommends
-
-    def suggests(self):
-        suggests = self.__reqprov(5049, 5051, 5050)
-        if not suggests:
-            suggests = self.__reqprov(1156, 1158, 1157, 0)
-        return suggests
-
-    def supplements(self):
-        return self.__reqprov(5052, 5054, 5053)
-
-    def enhances(self):
-        return self.__reqprov(5055, 5057, 5506)
-
     def is_src(self):
         # SOURCERPM = 1044
         return self.gettag(1044) is None
@@ -287,9 +250,9 @@ class RpmQuery(packagequery.PackageQuery, packagequery.PackageQueryResult):
 
     def canonname(self):
         if self.is_nosrc():
-            arch = b'nosrc'
+            arch = 'nosrc'
         elif self.is_src():
-            arch = b'src'
+            arch = 'src'
         else:
             arch = self.arch()
         return RpmQuery.filename(self.name(), None, self.version(), self.release(), arch)
@@ -322,24 +285,12 @@ class RpmQuery(packagequery.PackageQuery, packagequery.PackageQueryResult):
         if ver1 == ver2:
             return 0
         res = 0
-        ver1 = decode_it(ver1)
-        ver2 = decode_it(ver2)
         while res == 0:
-            # remove all leading non alphanumeric or tilde chars
-            ver1 = re.sub('^[^a-zA-Z0-9~]*', '', ver1)
-            ver2 = re.sub('^[^a-zA-Z0-9~]*', '', ver2)
-            if ver1.startswith('~') or ver2.startswith('~'):
-                if not ver1.startswith('~'):
-                    return 1
-                elif not ver2.startswith('~'):
-                    return -1
-                ver1 = ver1[1:]
-                ver2 = ver2[1:]
-                continue
-
+            # remove all leading non alphanumeric chars
+            ver1 = re.sub('^[^a-zA-Z0-9]*', '', ver1)
+            ver2 = re.sub('^[^a-zA-Z0-9]*', '', ver2)
             if not (len(ver1) and len(ver2)):
                 break
-
             # check if we have a digits segment
             mo1 = re.match('(\d+)', ver1)
             mo2 = re.match('(\d+)', ver2)
@@ -374,17 +325,18 @@ class RpmQuery(packagequery.PackageQuery, packagequery.PackageQueryResult):
 
     @staticmethod
     def filename(name, epoch, version, release, arch):
-        return b'%s-%s-%s.%s.rpm' % (name, version, release, arch)
+        return '%s-%s-%s.%s.rpm' % (name, version, release, arch)
 
-def unpack_string(data, encoding=None):
+def unpack_string(data):
     """unpack a '\\0' terminated string from data"""
-    idx = data.find(b'\0')
-    if idx == -1:
-        raise ValueError('illegal string: not \\0 terminated')
-    data = data[:idx]
-    if encoding is not None:
-        data = data.decode(encoding)
-    return data
+    val = ''
+    for c in data:
+        c, = struct.unpack('!c', c)
+        if c == '\0':
+            break
+        else:
+            val += c
+    return val
 
 if __name__ == '__main__':
     import sys
